package com.ruoyi.framework.web.service;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.SmUser;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.framework.security.context.AuthenticationContextHolder;
import com.ruoyi.ss.user.service.ISmUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.service.ISysUserService;

/**
 * 用户验证处理
 *
 * @author ruoyi
 */
@Service
@Primary
public class UserDetailsServiceImpl implements UserDetailsService
{
    private static final Logger log = LoggerFactory.getLogger(UserDetailsServiceImpl.class);

    @Autowired
    private ISysUserService userService;

    @Autowired
    private SysPasswordService passwordService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private ISmUserService smUserService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException
    {
        /** 获取到用户类型*/
        Authentication authentication = AuthenticationContextHolder.getContext();
        if (authentication != null && authentication.getDetails() != null) {
            String userType = (String) authentication.getDetails();
            if(Constants.USER_TYPE_PC.equals(userType)){
                // PC 用户登录
                SysUser user = userService.selectUserByUserName(username);
                this.checkUser(user, username);
                passwordService.validate(user);
                return createLoginUser(user);
            } else if(Constants.USER_TYPE_WX.equals(userType)) {
                // app 用户微信登录
                SmUser user = smUserService.selectUserByPhone(username);
                this.checkUser(user, username);
                return createLoginUser(user);
            } else if(Constants.USER_TYPE_APP.equals(userType)) {
                // app 用户手机号密码登录
                SmUser user = smUserService.selectUserByPhone(username);
                this.checkUser(user, username);
                passwordService.validate(user);
                return createLoginUser(user);
            } else {
                throw new ServiceException("不支持的登录方式");
            }

        } else {
            log.info("用户类型未正常获取");
            throw new ServiceException(MessageUtils.message("user.not.type"));
        }
    }

    /**
     * 校验用户账号是否正常
     */
    private void checkUser(SysUser user, String username) {
        if (StringUtils.isNull(user))
        {
            log.info("登录用户：{} 不存在.", username);
            throw new ServiceException(MessageUtils.message("user.not.exists"));
        }
        else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
        {
            log.info("登录用户：{} 已被删除.", username);
            throw new ServiceException(MessageUtils.message("user.password.delete"));
        }
        else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
        {
            log.info("登录用户：{} 已被停用.", username);
            throw new ServiceException(MessageUtils.message("user.blocked"));
        }
    }

    /**
     * 校验用户账号是否正常
     */
    private void checkUser(SmUser user, String username) {
        if (StringUtils.isNull(user))
        {
            log.info("登录用户：{} 不存在.", username);
            throw new ServiceException(MessageUtils.message("user.not.exists"));
        }
        else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
        {
            log.info("登录用户：{} 已被删除.", username);
            throw new ServiceException(MessageUtils.message("user.password.delete"));
        }
        else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
        {
            log.info("登录用户：{} 已被停用.", username);
            throw new ServiceException(MessageUtils.message("user.blocked"));
        }
    }

    public UserDetails createLoginUser(SysUser user)
    {
        return new LoginUser(user.getUserId(), user.getDeptId(), user, permissionService.getMenuPermission(user));
    }

    public UserDetails createLoginUser(SmUser user) {
        return new LoginUser(user.getUserId(), user);
    }
}
