package com.ruoyi.ss.device.service.impl;

import com.ruoyi.common.core.domain.BaseValidator;
import com.ruoyi.common.core.domain.ValidateResult;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.ss.device.domain.DeviceBO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.store.service.StoreValidator;
import com.ruoyi.ss.user.service.UserValidator;
import org.apache.commons.lang.StringUtils;
import org.eclipse.jetty.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;

/**
 * @author wjh
 * 2024/4/30
 */
@Service
public class DeviceValidatorImpl extends BaseValidator implements DeviceValidator {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private StoreValidator storeValidator;

    @Autowired
    private UserValidator userValidator;

    /**
     * 设备编号是否存在
     *
     * @param deviceNos
     */
    @Override
    public boolean isExistNo(List<String> deviceNos) {
        if (CollectionUtils.isEmptyElement(deviceNos)) {
            return true;
        }
        DeviceQuery query = new DeviceQuery();
        query.setDeviceNos(deviceNos);
        return deviceService.selectCount(query) == new HashSet<>(deviceNos).size();
    }

    /**
     * 判断设备是否属于指定用户
     *
     * @param deviceId 设备id
     * @param userId   用户id
     */
    @Override
    public boolean isBelong(Long deviceId, Long userId) {
        if (deviceId == null) {
            return false;
        }
        DeviceVO device = deviceService.selectSmDeviceByDeviceId(deviceId);
        return device != null && Objects.equals(device.getUserId(), userId);
    }

    /**
     * 创建前校验
     *
     * @param data
     * @return
     */
    @Override
    public ValidateResult preCreate(DeviceBO data) {
        if (data == null) {
            return error("数据不允许为空");
        }
        return success();
    }

    /**
     * 更新前校验
     *
     * @param data
     */
    @Override
    public ValidateResult preUpdate(DeviceBO data) {
        if (data == null) {
            return error("数据不允许为空");
        }
        return success();
    }

    /**
     * APP更新前校验
     *
     * @param data
     */
    @Override
    public ValidateResult preUpdateByApp(DeviceBO data) {
        if (data == null) {
            return error("数据不允许为空");
        }

        if (data.getStoreId() != null && !storeValidator.isExist(Collections.singletonList(data.getStoreId()))) {
            return error("店铺不存在");
        }

        Long userId = SecurityUtils.getUserId();

        // 判断是否有权限更改
        if (!isBelong(data.getDeviceId(), userId)) {
            return error("这不是您的设备，您无权更改");
        }

        return success();
    }

    @Override
    public boolean isRepeatSn(Long deviceId, String sn) {
        if (StringUtil.isBlank(sn)) {
            return false;
        }
        DeviceVO device = deviceService.selectByDeviceNo(sn);
        return device != null && !Objects.equals(deviceId, device.getDeviceId());
    }

    @Override
    public boolean isRepeatMac(Long deviceId, String mac) {
        if (StringUtils.isBlank(mac)) {
            return false;
        }
        DeviceVO device = deviceService.selectByMac(mac);
        return device != null && !Objects.equals(deviceId, device.getDeviceId());
    }
}
