package com.ruoyi.ss.transactionBill.service.impl;

import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.account.domain.AccountQuery;
import com.ruoyi.ss.account.domain.AccountVO;
import com.ruoyi.ss.account.domain.enums.AccountType;
import com.ruoyi.ss.account.service.AccountService;
import com.ruoyi.ss.channel.domain.ChannelVO;
import com.ruoyi.ss.channel.service.ChannelService;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.suit.domain.SuitVo;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.bo.RechargeBO;
import com.ruoyi.ss.transactionBill.domain.bo.WithdrawBO;
import com.ruoyi.ss.transactionBill.domain.dto.WithdrawDTO;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillPayType;
import com.ruoyi.ss.transactionBill.service.TransactionBillConverter;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author wjh
 * 2024/7/28
 */
@Service
public class TransactionBillConverterImpl implements TransactionBillConverter {

    @Autowired
    private DeviceService deviceService;
    @Autowired
    private ISmUserService userService;
    @Autowired
    private SuitService suitService;
    @Autowired
    private ChannelService channelService;
    @Autowired
    private AccountService accountService;

    /**
     * 将参数转为充值BO
     *
     * @param data
     */
    @Override
    public RechargeBO toRechargeBO(TransactionBill data) {
        if (data == null) {
            return null;
        }

        DeviceVO device = deviceService.selectByDeviceNo(data.getDeviceNo());
        ServiceUtil.assertion(device == null, "设备不存在");


        SmUserVo mch = userService.selectSmUserByUserId(device.getUserId());
        ServiceUtil.assertion(mch == null, "商户不存在");

        SuitVo suit = suitService.selectSuitBySuitId(data.getSuitId());
        ServiceUtil.assertion(suit == null, "套餐不存在");

        SmUserVo user = userService.selectSmUserByUserId(data.getUserId());
        ServiceUtil.assertion(user == null, "用户不存在");
        ServiceUtil.assertion(user.getWxOpenId() == null, "用户微信openId为空");

        ChannelVO channel = channelService.selectSmChannelByChannelId(data.getChannelId());
        ServiceUtil.assertion(channel == null, "支付渠道不存在");

        RechargeBO bo = new RechargeBO();
        bo.setDevice(device);
        bo.setUser(user);
        bo.setChannel(channel);
        bo.setSuit(suit);
        bo.setMch(mch);
        bo.setParams(data);
        return null;
    }

    /**
     * 将参数转为提现BO
     *
     * @param dto
     */
    @Override
    public WithdrawBO toWithdrawBO(WithdrawDTO dto) {
        if (dto == null) {
            return null;
        }

        // 用户
        SmUserVo user = userService.selectSmUserByUserId(dto.getUserId());
        ServiceUtil.assertion(user == null, "用户不存在");

        // 渠道
        ChannelVO channel = channelService.selectSmChannelByChannelId(dto.getChannelId());
        ServiceUtil.assertion(channel == null, "提现渠道不存在");

        // 获取收款账户
        AccountType accountType = AccountType.parseByChannel(channel.getChannelId());
        AccountQuery accountQuery = new AccountQuery();
        accountQuery.setUserId(user.getUserId());
        accountQuery.setAccountType(accountType.getType());
        AccountVO account = accountService.selectOne(accountQuery);
        ServiceUtil.assertion(account == null, "收款账户不存在");

        WithdrawBO bo = new WithdrawBO();
        bo.setDto(dto);
        bo.setUser(user);
        bo.setChannel(channel);
        bo.setAccount(account);
        return bo;
    }
}
