package com.ruoyi.web.controller.app;

import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.domain.dto.BillRefundDTO;
import com.ruoyi.ss.transactionBill.domain.dto.WithdrawDTO;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillGroupBy;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.service.TransactionAssembler;
import com.ruoyi.ss.transactionBill.service.TransactionBillConverter;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.transactionBill.service.TransactionBillValidator;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 充值记录Controller
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Api(tags = "充值/提现记录")
@RestController
@RequestMapping("/app/bill")
public class AppTransactionBillController extends BaseController
{
    @Autowired
    private TransactionBillService smTransactionBillService;

    @Autowired
    private TransactionAssembler transactionAssembler;

    @Autowired
    private TransactionBillValidator transactionBillValidator;

    @Autowired
    private DeviceValidator deviceValidator;

    @Autowired
    private TransactionBillConverter transactionBillConverter;

    /**
     * 查询充值记录列表
     */
    @ApiOperation("查询本人订单列表")
    @GetMapping("/rechargeList")
    @JsonView(JsonViewProfile.App.class)
    public TableDataInfo rechargeList(TransactionBillQuery smTransactionBill)
    {
        smTransactionBill.setUserId(getUserId());
        smTransactionBill.setType(TransactionBillType.RECHARGE.getType());
        startPage();
        List<TransactionBillVO> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        transactionAssembler.assembleChannelName(list);
        return getDataTable(list);
    }

    /**
     * 查询提现记录列表
     */
    @ApiOperation("查询本人提现记录列表")
    @GetMapping("/withdrawList")
    public TableDataInfo withdrawList(TransactionBillQuery smTransactionBill)
    {
        startPage();
        smTransactionBill.setUserId(getUserId());
        smTransactionBill.setType(TransactionBillType.WITHDRAW.getType());
        List<TransactionBillVO> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        return getDataTable(list);
    }

    /**
     * 查询收支列表
     */
//    @MchRequired
    @ApiOperation("查询本人收支列表")
    @GetMapping("/list")
    @JsonView(JsonViewProfile.AppMch.class)
    public TableDataInfo list(TransactionBillQuery smTransactionBill)
    {
        startPage();
        smTransactionBill.setUserId(getUserId());
        List<TransactionBillVO> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        transactionAssembler.assembleChannelName(list);
        return getDataTable(list);
    }

    @ApiOperation("查询本人收款的订单列表")
    @GetMapping("/incomeList")
    @JsonView(JsonViewProfile.AppMch.class)
    public TableDataInfo incomeList(TransactionBillQuery smTransactionBill)
    {
        startPage();
        smTransactionBill.setMchId(getUserId());
        smTransactionBill.setType(TransactionBillType.RECHARGE.getType());
        List<TransactionBillVO> list = smTransactionBillService.selectSmTransactionBillList(smTransactionBill);
        transactionAssembler.assembleChannelName(list);
        return getDataTable(list);
    }

    @ApiOperation("获取订单详情")
    @GetMapping("/{billId}")
    @JsonView(JsonViewProfile.AppUser.class)
    public AjaxResult detail(@PathVariable Long billId) {
        ServiceUtil.assertion(transactionBillValidator.preGetDetailByApp(billId, getUserId()));
        TransactionBillVO bill = smTransactionBillService.selectSmTransactionBillByBillId(billId);
        transactionAssembler.assembleChannelName(Collections.singletonList(bill));
        return AjaxResult.success(bill);
    }

//    @MchRequired
    @ApiOperation("获取商户统计信息")
    @JsonView(JsonViewProfile.AppMch.class)
    @GetMapping("/landlordCount")
    public AjaxResult landlordCount(@Validated({ValidGroup.Query.class}) TransactionBillQuery dto) {
        Date now = new Date();
        if (dto.getEndDate() == null) {
            dto.setEndDate(now);
        }
        // 如果按日查询
        if (Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_date.name())) {
            // 查询的月份是当前月份，并且当前日期小于7日，则查询最近7天的数据，否则查询当月第一天开始的数据
//            if (Objects.equals(dto.getYear(), DateUtils.getYear(now))
//                    && Objects.equals(dto.getMonth(), DateUtils.getMonth(now))
//                    && DateUtils.getDay(now) < 7) {
//                // 七天前
//                dto.setStartDate(DateUtils.addDays(dto.getEndDate(), -6));
//                dto.setYear(null);
//                dto.setMonth(null);
//            } else {
//                dto.setStartDate(DateUtils.getMonthFirstDate(dto.getYear(), dto.getMonth()));
//                dto.setEndDate(DateUtils.getMonthLastDate(dto.getYear(), dto.getMonth()));
//            }

            // 以上是旧的需求，防止要改回去
            // 以下是新的需求，如果按日分组，则查询当月第一天到最后一天的数据，并补全中间缺少的数据
            dto.setStartDate(DateUtils.getMonthFirstDate(dto.getYear(), dto.getMonth()));
            dto.setEndDate(DateUtils.getMonthLastDate(dto.getYear(), dto.getMonth()));
        }
        dto.setMchId(getUserId());
        dto.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        AjaxResult ajax = AjaxResult.success(smTransactionBillService.selectLandlordCount(dto));

        // 总收入
        TransactionBillQuery allQuery = new TransactionBillQuery();
        allQuery.setMchId(getUserId());
        List<BillCountVo> totalList = smTransactionBillService.selectCount(allQuery);
        if (CollectionUtils.isEmpty(totalList)) {
            ajax.put("totalRecharge", 0);
        } else {
            BigDecimal total = totalList.get(0).getRecharge();
            ajax.put("totalRecharge", total == null ? 0 : total);
        }

        return ajax;
    }

    @ApiOperation("创建订单")
    @PostMapping("/recharge")
    public AjaxResult addRecharge(@RequestBody @Validated({ValidGroup.Recharge.class}) TransactionBill body) {
        body.setUserId(getUserId());
        body.setChannelId(1L);  // 微信支付
        return AjaxResult.success("操作成功", smTransactionBillService.addOrder(transactionBillConverter.toRechargeBO(body)));
    }

    @ApiOperation("取消充值订单")
    @PutMapping("/recharge/cancel/{billNo}")
    public AjaxResult cancelRecharge(@PathVariable @ApiParam("订单编号") String billNo) {
        return AjaxResult.success(smTransactionBillService.cancelRecharge(billNo, TransactionBillStatus.CANCELED));
    }

//    @MchRequired
    @ApiOperation("提现申请")
    @PostMapping("/withdraw")
    public AjaxResult withdraw(@RequestBody @Validated({ValidGroup.Withdraw.class}) WithdrawDTO dto) {
        dto.setUserId(getUserId());
        return AjaxResult.success("操作成功", smTransactionBillService.addWithdraw(transactionBillConverter.toWithdrawBO(dto)));
    }

    @ApiOperation("查询设备充值失败列表")
    @GetMapping("/recharge/device/fail/list")
    public AjaxResult selectDeviceRechargeFailList() {
        return AjaxResult.success(smTransactionBillService.selectDeviceRechargeFailList(getUserId()));
    }

//    @MchRequired
    @ApiOperation("获取设备充值列表")
    @GetMapping("/rechargeListByDevice")
    @JsonView(JsonViewProfile.AppMch.class)
    public TableDataInfo getDeviceRechargeList(@RequestParam @ApiParam("设备id") Long deviceId) {
        ServiceUtil.assertion(!deviceValidator.isBelong(deviceId, getUserId()), "这不是您的设备，无法查询");
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceId(deviceId);
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        startPage();
        return getDataTable(smTransactionBillService.selectSmTransactionBillList(query));
    }

    @ApiOperation("蓝牙充值成功回调")
    @GetMapping("/recharge/{billNo}/bluetoothSuccess")
    public AjaxResult bluetoothRechargeSuccess(@PathVariable String billNo) {
        return success(smTransactionBillService.bluetoothRechargeSuccess(billNo));
    }

    @ApiOperation("订单退款")
    @PutMapping("/refund")
    public AjaxResult refund(@RequestBody @Validated BillRefundDTO dto) {
        // 判断用户是否订单收款人
        TransactionBillVO bill = smTransactionBillService.selectSmTransactionBillByBillId(dto.getBillId());
        if (bill == null) {
            return error("订单不存在");
        }
        if (!Objects.equals(bill.getMchId(), getUserId())) {
            return error("您无权操作退款");
        }
        LoginUser loginUser = getLoginUser();
        dto.setUserName(loginUser.getUsername());
        dto.setUserType(loginUser.getLoginType().getType());
        dto.setUserId(getUserId());
        return toAjax(smTransactionBillService.refund(dto));
    }

    @ApiOperation("刷新支付结果")
    @PutMapping("/{billNo}/refreshPayResult")
    public AjaxResult refreshPayResult(@PathVariable String billNo ) {
        return toAjax(smTransactionBillService.refreshPayResult(billNo));
    }

    @ApiOperation("获取用户提现手续费")
    @GetMapping("/getWithdrawService")
    public AjaxResult getWithdrawService(@RequestParam @ApiParam("渠道ID") Long channelId) {
        return success(smTransactionBillService.getUserWithdrawService(getUserId(), channelId));
    }

}
