package com.ruoyi.web.controller.app;

import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.user.domain.bo.SmUserBO;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.ss.user.service.UserAssembler;
import com.ruoyi.system.domain.enums.verificationCode.CodeBusinessType;
import com.ruoyi.system.service.IVerificationCodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collections;
import java.util.List;

/**
 * 用户信息Controller
 * @author 辉
 * 2024/3/6
 */
@Api(tags = "用户信息")
@RestController
@RequestMapping("/app/user")
public class AppUserController extends BaseController {

    @Autowired
    private ISmUserService userService;

    @Autowired
    private IVerificationCodeService verificationCodeService;

    @Autowired
    private UserAssembler userAssembler;

    @ApiOperation("获取当前登录前台用户的信息")
    @GetMapping("/userInfo")
    @JsonView(JsonViewProfile.App.class)
    public AjaxResult userInfo() {
        // 查询
        SmUserVo user = userService.selectSmUserByUserId(getUserId());
        List<SmUserVo> list = Collections.singletonList(user);
        userService.desensitization(list);
        userAssembler.assembleStoreCount(list);
        userAssembler.assembleDeviceCount(list);
        return AjaxResult.success(user);
    }

    @ApiOperation("修改用户类型")
    @PutMapping("/changeType")
    public AjaxResult changeRole(String userType) {
        return AjaxResult.success(userService.changeType(getUserId(), UserType.parse(userType)));
    }

    @ApiOperation("修改密码")
    @PutMapping("/updatePassword")
    public AjaxResult updatePassword(SmUserBO bo) {
        ServiceUtil.assertion(!verificationCodeService.checkCode(bo.getPhonenumber(), bo.getMobileCode(), CodeBusinessType.UPDATE_PASSWORD, true),
                "短信验证码错误，请重试");
        return AjaxResult.success(userService.updatePassword(getUserId(), bo.getPassword()));
    }


}
